@@ -1,4 +1,4 @@
-/*	$NetBSD: el.c,v 1.101 2022/10/30 19:11:31 christos Exp $	*/
+/*	$NetBSD: el.c,v 1.45 2008/04/05 15:53:28 christos Exp $	*/
 
 /*-
  * Copyright (c) 1992, 1993
@@ -37,82 +37,41 @@
 #if 0
 static char sccsid[] = "@(#)el.c	8.2 (Berkeley) 1/3/94";
 #else
-__RCSID("$NetBSD: el.c,v 1.101 2022/10/30 19:11:31 christos Exp $");
+__RCSID("$NetBSD: el.c,v 1.45 2008/04/05 15:53:28 christos Exp $");
 #endif
 #endif /* not lint && not SCCSID */
 
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 4096
-#endif
-
 /*
  * el.c: EditLine interface functions
  */
 #include <sys/types.h>
 #include <sys/param.h>
-#include <ctype.h>
-#include <langinfo.h>
-#include <locale.h>
-#include <stdarg.h>
-#include <stdlib.h>
 #include <string.h>
-
+#include <stdlib.h>
+#include <stdarg.h>
 #include "el.h"
-#include "parse.h"
-#include "read.h"
-
-#ifndef HAVE_SECURE_GETENV
-#	ifdef HAVE___SECURE_GETENV
-#		define secure_getenv __secure_getenv
-#		define HAVE_SECURE_GETENV 1
-#	else
-#		ifdef HAVE_ISSETUGID
-#			include <unistd.h>
-#		else
-#			undef issetugid
-#			define issetugid() 1
-#		endif
-#	endif
-#endif
-
-#ifndef HAVE_SECURE_GETENV
-char *secure_getenv(char const *name)
-{
-	if (issetugid())
-		return 0;
-	return getenv(name);
-}
-#endif
 
 /* el_init():
  *	Initialize editline and set default parameters.
  */
-EditLine *
+public EditLine *
 el_init(const char *prog, FILE *fin, FILE *fout, FILE *ferr)
 {
-    return el_init_fd(prog, fin, fout, ferr, fileno(fin), fileno(fout),
-	fileno(ferr));
-}
 
-libedit_private EditLine *
-el_init_internal(const char *prog, FILE *fin, FILE *fout, FILE *ferr,
-    int fdin, int fdout, int fderr, int flags)
-{
-	EditLine *el = el_calloc(1, sizeof(*el));
+	EditLine *el = (EditLine *) el_malloc(sizeof(EditLine));
 
 	if (el == NULL)
-		return NULL;
+		return (NULL);
+
+	memset(el, 0, sizeof(EditLine));
 
 	el->el_infile = fin;
 	el->el_outfile = fout;
 	el->el_errfile = ferr;
 
-	el->el_infd = fdin;
-	el->el_outfd = fdout;
-	el->el_errfd = fderr;
+	el->el_infd = fileno(fin);
 
-	el->el_prog = wcsdup(ct_decode_string(prog, &el->el_scratch));
-	if (el->el_prog == NULL) {
+	if ((el->el_prog = el_strdup(prog)) == NULL) {
 		el_free(el);
 		return NULL;
 	}
@@ -120,14 +79,14 @@ el_init_internal(const char *prog, FILE *fin, FILE *fout, FILE *ferr,
 	/*
          * Initialize all the modules. Order is important!!!
          */
-	el->el_flags = flags;
+	el->el_flags = 0;
 
-	if (terminal_init(el) == -1) {
+	if (term_init(el) == -1) {
 		el_free(el->el_prog);
 		el_free(el);
 		return NULL;
 	}
-	(void) keymacro_init(el);
+	(void) key_init(el);
 	(void) map_init(el);
 	if (tty_init(el) == -1)
 		el->el_flags |= NO_TTY;
@@ -136,25 +95,16 @@ el_init_internal(const char *prog, FILE *fin, FILE *fout, FILE *ferr,
 	(void) hist_init(el);
 	(void) prompt_init(el);
 	(void) sig_init(el);
-	(void) literal_init(el);
-	if (read_init(el) == -1) {
-		el_end(el);
-		return NULL;
-	}
-	return el;
+	(void) read_init(el);
+
+	return (el);
 }
 
-EditLine *
-el_init_fd(const char *prog, FILE *fin, FILE *fout, FILE *ferr,
-    int fdin, int fdout, int fderr)
-{
-	return el_init_internal(prog, fin, fout, ferr, fdin, fdout, fderr, 0);
-}
 
 /* el_end():
  *	Clean up.
  */
-void
+public void
 el_end(EditLine *el)
 {
 
@@ -163,93 +113,58 @@ el_end(EditLine *el)
 
 	el_reset(el);
 
-	terminal_end(el);
-	keymacro_end(el);
+	term_end(el);
+	key_end(el);
 	map_end(el);
-	if (!(el->el_flags & NO_TTY))
-		tty_end(el, TCSAFLUSH);
+	tty_end(el);
 	ch_end(el);
-	read_end(el);
 	search_end(el);
 	hist_end(el);
 	prompt_end(el);
 	sig_end(el);
-	literal_end(el);
 
-	el_free(el->el_prog);
-	el_free(el->el_visual.cbuff);
-	el_free(el->el_visual.wbuff);
-	el_free(el->el_scratch.cbuff);
-	el_free(el->el_scratch.wbuff);
-	el_free(el->el_lgcyconv.cbuff);
-	el_free(el->el_lgcyconv.wbuff);
-	el_free(el);
+	el_free((ptr_t) el->el_prog);
+	el_free((ptr_t) el);
 }
 
 
 /* el_reset():
  *	Reset the tty and the parser
  */
-void
+public void
 el_reset(EditLine *el)
 {
 
 	tty_cookedmode(el);
-	ch_reset(el);		/* XXX: Do we want that? */
+	ch_reset(el, 0);		/* XXX: Do we want that? */
 }
 
 
 /* el_set():
  *	set the editline parameters
  */
-int
-el_wset(EditLine *el, int op, ...)
+public int
+el_set(EditLine *el, int op, ...)
 {
 	va_list ap;
 	int rv = 0;
 
 	if (el == NULL)
-		return -1;
+		return (-1);
 	va_start(ap, op);
 
 	switch (op) {
 	case EL_PROMPT:
-	case EL_RPROMPT: {
-		el_pfunc_t p = va_arg(ap, el_pfunc_t);
-
-		rv = prompt_set(el, p, 0, op, 1);
+	case EL_RPROMPT:
+		rv = prompt_set(el, va_arg(ap, el_pfunc_t), op);
 		break;
-	}
-
-	case EL_RESIZE: {
-		el_zfunc_t p = va_arg(ap, el_zfunc_t);
-		void *arg = va_arg(ap, void *);
-		rv = ch_resizefun(el, p, arg);
-		break;
-	}
-
-	case EL_ALIAS_TEXT: {
-		el_afunc_t p = va_arg(ap, el_afunc_t);
-		void *arg = va_arg(ap, void *);
-		rv = ch_aliasfun(el, p, arg);
-		break;
-	}
-
-	case EL_PROMPT_ESC:
-	case EL_RPROMPT_ESC: {
-		el_pfunc_t p = va_arg(ap, el_pfunc_t);
-		int c = va_arg(ap, int);
-
-		rv = prompt_set(el, p, (wchar_t)c, op, 1);
-		break;
-	}
 
 	case EL_TERMINAL:
-		rv = terminal_set(el, va_arg(ap, char *));
+		rv = term_set(el, va_arg(ap, char *));
 		break;
 
 	case EL_EDITOR:
-		rv = map_set_editor(el, va_arg(ap, wchar_t *));
+		rv = map_set_editor(el, va_arg(ap, char *));
 		break;
 
 	case EL_SIGNAL:
@@ -262,39 +177,40 @@ el_wset(EditLine *el, int op, ...)
 	case EL_BIND:
 	case EL_TELLTC:
 	case EL_SETTC:
+	case EL_GETTC:
 	case EL_ECHOTC:
 	case EL_SETTY:
 	{
-		const wchar_t *argv[20];
+		const char *argv[20];
 		int i;
 
-		for (i = 1; i < (int)__arraycount(argv); i++)
-			if ((argv[i] = va_arg(ap, wchar_t *)) == NULL)
+		for (i = 1; i < 20; i++)
+			if ((argv[i] = va_arg(ap, char *)) == NULL)
 				break;
 
 		switch (op) {
 		case EL_BIND:
-			argv[0] = L"bind";
+			argv[0] = "bind";
 			rv = map_bind(el, i, argv);
 			break;
 
 		case EL_TELLTC:
-			argv[0] = L"telltc";
-			rv = terminal_telltc(el, i, argv);
+			argv[0] = "telltc";
+			rv = term_telltc(el, i, argv);
 			break;
 
 		case EL_SETTC:
-			argv[0] = L"settc";
-			rv = terminal_settc(el, i, argv);
+			argv[0] = "settc";
+			rv = term_settc(el, i, argv);
 			break;
 
 		case EL_ECHOTC:
-			argv[0] = L"echotc";
-			rv = terminal_echotc(el, i, argv);
+			argv[0] = "echotc";
+			rv = term_echotc(el, i, argv);
 			break;
 
 		case EL_SETTY:
-			argv[0] = L"setty";
+			argv[0] = "setty";
 			rv = tty_stty(el, i, argv);
 			break;
 
@@ -308,8 +224,8 @@ el_wset(EditLine *el, int op, ...)
 
 	case EL_ADDFN:
 	{
-		wchar_t *name = va_arg(ap, wchar_t *);
-		wchar_t *help = va_arg(ap, wchar_t *);
+		char *name = va_arg(ap, char *);
+		char *help = va_arg(ap, char *);
 		el_func_t func = va_arg(ap, el_func_t);
 
 		rv = map_addfunc(el, name, help, func);
@@ -319,22 +235,12 @@ el_wset(EditLine *el, int op, ...)
 	case EL_HIST:
 	{
 		hist_fun_t func = va_arg(ap, hist_fun_t);
-		void *ptr = va_arg(ap, void *);
+		ptr_t ptr = va_arg(ap, char *);
 
 		rv = hist_set(el, func, ptr);
-		if (MB_CUR_MAX == 1)
-			el->el_flags &= ~NARROW_HISTORY;
 		break;
 	}
 
-	case EL_SAFEREAD:
-		if (va_arg(ap, int))
-			el->el_flags |= FIXIO;
-		else
-			el->el_flags &= ~FIXIO;
-		rv = 0;
-		break;
-
 	case EL_EDITMODE:
 		if (va_arg(ap, int))
 			el->el_flags &= ~EDIT_DISABLED;
@@ -346,7 +252,7 @@ el_wset(EditLine *el, int op, ...)
 	case EL_GETCFN:
 	{
 		el_rfunc_t rc = va_arg(ap, el_rfunc_t);
-		rv = el_read_setfn(el->el_read, rc);
+		rv = el_read_setfn(el, rc);
 		break;
 	}
 
@@ -391,11 +297,9 @@ el_wset(EditLine *el, int op, ...)
 			break;
 		case 1:
 			el->el_outfile = fp;
-			el->el_outfd = fileno(fp);
 			break;
 		case 2:
 			el->el_errfile = fp;
-			el->el_errfd = fileno(fp);
 			break;
 		default:
 			rv = -1;
@@ -407,7 +311,7 @@ el_wset(EditLine *el, int op, ...)
 	case EL_REFRESH:
 		re_clear_display(el);
 		re_refresh(el);
-		terminal__flush(el);
+		term__flush();
 		break;
 
 	default:
@@ -416,15 +320,15 @@ el_wset(EditLine *el, int op, ...)
 	}
 
 	va_end(ap);
-	return rv;
+	return (rv);
 }
 
 
 /* el_get():
  *	retrieve the editline parameters
  */
-int
-el_wget(EditLine *el, int op, ...)
+public int
+el_get(EditLine *el, int op, ...)
 {
 	va_list ap;
 	int rv;
@@ -436,22 +340,12 @@ el_wget(EditLine *el, int op, ...)
 
 	switch (op) {
 	case EL_PROMPT:
-	case EL_RPROMPT: {
-		el_pfunc_t *p = va_arg(ap, el_pfunc_t *);
-		rv = prompt_get(el, p, 0, op);
+	case EL_RPROMPT:
+		rv = prompt_get(el, va_arg(ap, el_pfunc_t *), op);
 		break;
-	}
-	case EL_PROMPT_ESC:
-	case EL_RPROMPT_ESC: {
-		el_pfunc_t *p = va_arg(ap, el_pfunc_t *);
-		wchar_t *c = va_arg(ap, wchar_t *);
-
-		rv = prompt_get(el, p, c, op);
-		break;
-	}
 
 	case EL_EDITOR:
-		rv = map_get_editor(el, va_arg(ap, const wchar_t **));
+		rv = map_get_editor(el, va_arg(ap, const char **));
 		break;
 
 	case EL_SIGNAL:
@@ -464,29 +358,57 @@ el_wget(EditLine *el, int op, ...)
 		rv = 0;
 		break;
 
-	case EL_SAFEREAD:
-		*va_arg(ap, int *) = (el->el_flags & FIXIO);
-		rv = 0;
-		break;
-
 	case EL_TERMINAL:
-		terminal_get(el, va_arg(ap, const char **));
+		term_get(el, va_arg(ap, const char **));
 		rv = 0;
 		break;
 
 	case EL_GETTC:
 	{
 		static char name[] = "gettc";
-		char *argv[3];
-		argv[0] = name;
-		argv[1] = va_arg(ap, char *);
-		argv[2] = va_arg(ap, void *);
-		rv = terminal_gettc(el, 3, argv);
+		char *argv[20];
+		int i;
+
+ 		for (i = 1; i < sizeof(argv) / sizeof(argv[0]); i++)
+			if ((argv[i] = va_arg(ap, char *)) == NULL)
+				break;
+
+		switch (op) {
+		case EL_GETTC:
+			argv[0] = name;
+			rv = term_gettc(el, i, argv);
+			break;
+
+		default:
+			rv = -1;
+			EL_ABORT((el->el_errfile, "Bad op %d\n", op));
+			break;
+		}
 		break;
 	}
 
+#if 0 /* XXX */
+	case EL_ADDFN:
+	{
+		char *name = va_arg(ap, char *);
+		char *help = va_arg(ap, char *);
+		el_func_t func = va_arg(ap, el_func_t);
+
+		rv = map_addfunc(el, name, help, func);
+		break;
+	}
+
+	case EL_HIST:
+		{
+			hist_fun_t func = va_arg(ap, hist_fun_t);
+			ptr_t ptr = va_arg(ap, char *);
+			rv = hist_set(el, func, ptr);
+		}
+		break;
+#endif /* XXX */
+
 	case EL_GETCFN:
-		*va_arg(ap, el_rfunc_t *) = el_read_getfn(el->el_read);
+		*va_arg(ap, el_rfunc_t *) = el_read_getfn(el);
 		rv = 0;
 		break;
 
@@ -496,7 +418,7 @@ el_wget(EditLine *el, int op, ...)
 		break;
 
 	case EL_UNBUFFERED:
-		*va_arg(ap, int *) = (el->el_flags & UNBUFFERED) != 0;
+		*va_arg(ap, int *) = (!(el->el_flags & UNBUFFERED));
 		rv = 0;
 		break;
 
@@ -530,97 +452,76 @@ el_wget(EditLine *el, int op, ...)
 	}
 	va_end(ap);
 
-	return rv;
+	return (rv);
 }
 
 
 /* el_line():
  *	Return editing info
  */
-const LineInfoW *
-el_wline(EditLine *el)
+public const LineInfo *
+el_line(EditLine *el)
 {
 
-	return (const LineInfoW *)(void *)&el->el_line;
+	return (const LineInfo *) (void *) &el->el_line;
 }
 
 
 /* el_source():
  *	Source a file
  */
-int
+public int
 el_source(EditLine *el, const char *fname)
 {
 	FILE *fp;
 	size_t len;
-	ssize_t slen;
 	char *ptr;
-	char *path = NULL;
-	const wchar_t *dptr;
-	int error = 0;
 
 	fp = NULL;
 	if (fname == NULL) {
+		static const char elpath[] = "/.editrc";
+#ifdef MAXPATHLEN
+		char path[MAXPATHLEN];
+#else
+		char path[4096];
+#endif
 
-		/* secure_getenv is guaranteed to be defined and do the right thing here */
-		/* because of the defines above which take into account issetugid, */
-		/* secure_getenv and __secure_getenv availability. */
-		if ((fname = secure_getenv("EDITRC")) == NULL) {
-			static const char elpath[] = "/.editrc";
-			size_t plen = sizeof(elpath);
-
-			if ((ptr = secure_getenv("HOME")) == NULL)
-				return -1;
-			plen += strlen(ptr);
-			if ((path = el_calloc(plen, sizeof(*path))) == NULL)
-				return -1;
-			(void)snprintf(path, plen, "%s%s", ptr,
-				elpath + (*ptr == '\0'));
-			fname = path;
-		}
-
+#ifdef HAVE_ISSETUGID
+		if (issetugid())
+			return (-1);
+#endif
+		if ((ptr = getenv("HOME")) == NULL)
+			return (-1);
+		if (strlcpy(path, ptr, sizeof(path)) >= sizeof(path))
+			return (-1);
+		if (strlcat(path, elpath, sizeof(path)) >= sizeof(path))
+			return (-1);
+		fname = path;
 	}
-	if (fname[0] == '\0')
-		return -1;
-
 	if (fp == NULL)
 		fp = fopen(fname, "r");
-	if (fp == NULL) {
-		el_free(path);
-		return -1;
+	if (fp == NULL)
+		return (-1);
+
+	while ((ptr = fgetln(fp, &len)) != NULL) {
+		if (len > 0 && ptr[len - 1] == '\n')
+			--len;
+		ptr[len] = '\0';
+		if (parse_line(el, ptr) == -1) {
+			(void) fclose(fp);
+			return (-1);
+		}
 	}
 
-	ptr = NULL;
-	len = 0;
-	while ((slen = getline(&ptr, &len, fp)) != -1) {
-		if (*ptr == '\n')
-			continue;	/* Empty line. */
-		if (slen > 0 && ptr[--slen] == '\n')
-			ptr[slen] = '\0';
-
-		dptr = ct_decode_string(ptr, &el->el_scratch);
-		if (!dptr)
-			continue;
-		/* loop until first non-space char or EOL */
-		while (*dptr != '\0' && iswspace(*dptr))
-			dptr++;
-		if (*dptr == '#')
-			continue;   /* ignore, this is a comment line */
-		if ((error = parse_line(el, dptr)) == -1)
-			break;
-	}
-	free(ptr);
-
-	el_free(path);
 	(void) fclose(fp);
-	return error;
+	return (0);
 }
 
 
 /* el_resize():
  *	Called from program when terminal is resized
  */
-void
+public void
 el_resize(EditLine *el)
 {
 	int lins, cols;
@@ -631,8 +532,8 @@ el_resize(EditLine *el)
 	(void) sigprocmask(SIG_BLOCK, &nset, &oset);
 
 	/* get the correct window size */
-	if (terminal_get_size(el, &lins, &cols))
-		terminal_change_size(el, lins, cols);
+	if (term_get_size(el, &lins, &cols))
+		term_change_size(el, lins, cols);
 
 	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
 }
@@ -641,38 +542,37 @@ el_resize(EditLine *el)
 /* el_beep():
  *	Called from the program to beep
  */
-void
+public void
 el_beep(EditLine *el)
 {
 
-	terminal_beep(el);
+	term_beep(el);
 }
 
 
 /* el_editmode()
  *	Set the state of EDIT_DISABLED from the `edit' command.
  */
-libedit_private int
+protected int
 /*ARGSUSED*/
-el_editmode(EditLine *el, int argc, const wchar_t **argv)
+el_editmode(EditLine *el, int argc, const char **argv)
 {
-	const wchar_t *how;
+	const char *how;
 
 	if (argv == NULL || argc != 2 || argv[1] == NULL)
-		return -1;
+		return (-1);
 
 	how = argv[1];
-	if (wcscmp(how, L"on") == 0) {
+	if (strcmp(how, "on") == 0) {
 		el->el_flags &= ~EDIT_DISABLED;
 		tty_rawmode(el);
-	} else if (wcscmp(how, L"off") == 0) {
+	} else if (strcmp(how, "off") == 0) {
 		tty_cookedmode(el);
 		el->el_flags |= EDIT_DISABLED;
 	}
 	else {
-		(void) fprintf(el->el_errfile, "edit: Bad value `%ls'.\n",
-		    how);
-		return -1;
+		(void) fprintf(el->el_errfile, "edit: Bad value `%s'.\n", how);
+		return (-1);
 	}
-	return 0;
+	return (0);
 }
